<?php

namespace Ranig;

use FML\Controls\Frame;
use FML\Controls\Quads\Quad_BgsPlayerCard;
use FML\Controls\Quads\Quad_Icons64x64_1;
use FML\Controls\Quads\Quad_UIConstruction_Buttons;
use FML\ManiaLink;
use FML\Script\Features\Paging;
use ManiaControl\Callbacks\CallbackListener;
use ManiaControl\Callbacks\CallbackManager;
use ManiaControl\Callbacks\Callbacks;
use ManiaControl\Commands\CommandListener;
use ManiaControl\Logger;
use ManiaControl\ManiaControl;
use ManiaControl\Manialinks\LabelLine;
use ManiaControl\Manialinks\ManialinkManager;
use ManiaControl\Manialinks\ManialinkPageAnswerListener;
use ManiaControl\Players\Player;
use ManiaControl\Players\PlayerManager;
use ManiaControl\Plugins\Plugin;
use ManiaControl\Statistics\StatisticCollector;
use ManiaControl\Statistics\StatisticManager;
use ManiaControl\Utils\Formatter;
use Maniaplanet\DedicatedServer\Structures\AbstractStructure;
use MCTeam\LocalRecordsPlugin;
use MCTeam\ServerRankingPlugin;

/**
 */
class ExtendedServerRankingPlugin implements ManialinkPageAnswerListener, Plugin, CallbackListener, CommandListener {
	/*
	 * Constants
	 */
	const PLUGIN_ID                     = 132;
	const PLUGIN_VERSION                = 5;
	const PLUGIN_NAME                   = 'Extended Server Ranking Plugin';
	const PLUGIN_AUTHOR                 = 'Ranig';
	const DEFAULT_LOCAL_RECORDS_PLUGIN  = 'MCTeam\LocalRecordsPlugin';
	const DEFAULT_SERVER_RANKING_PLUGIN = 'MCTeam\ServerRankingPlugin';
	const ACTION_OPEN_RANKINGS          = 'ExtendedServerRankingPlugin.OpenRankings';

	/*
	 * Private properties
	 */
	/** @var ManiaControl $maniaControl * */
	private $maniaControl = null;

	/** @var ServerRankingPlugin $ranking * */
	private $ranking = null;

	/** @var LocalRecordsPlugin $records * */
	private $records = null;

	/**
	 * @see \ManiaControl\Plugins\Plugin::prepare()
	 */
	public static function prepare(ManiaControl $maniaControl) {
	}

	/**
	 * @see \ManiaControl\Plugins\Plugin::getId()
	 */
	public static function getId() {
		return self::PLUGIN_ID;
	}

	/**
	 * @see \ManiaControl\Plugins\Plugin::getName()
	 */
	public static function getName() {
		return self::PLUGIN_NAME;
	}

	/**
	 * @see \ManiaControl\Plugins\Plugin::getVersion()
	 */
	public static function getVersion() {
		return self::PLUGIN_VERSION;
	}

	/**
	 * @see \ManiaControl\Plugins\Plugin::getAuthor()
	 */
	public static function getAuthor() {
		return self::PLUGIN_AUTHOR;
	}

	/**
	 * @see \ManiaControl\Plugins\Plugin::getDescription()
	 */
	public static function getDescription() {
		return "ExtendedServerRanking Plugin, additional server ranking utilities";
	}

	/**
	 * @see \ManiaControl\Plugins\Plugin::load()
	 */
	public function load(ManiaControl $maniaControl) {
		$this->maniaControl = $maniaControl;

		$this->ranking = $this->maniaControl->getPluginManager()->getPlugin(self::DEFAULT_SERVER_RANKING_PLUGIN);
		$this->records = $this->maniaControl->getPluginManager()->getPlugin(self::DEFAULT_LOCAL_RECORDS_PLUGIN);
		
		if (!$this->ranking) {
			$this->maniaControl->getChat()->sendError('[ExtendedServerRankingPlugin] You need to activate ' . self::DEFAULT_SERVER_RANKING_PLUGIN);
		} else {
			$this->maniaControl->getCommandManager()->unregisterCommandListener($this->ranking);

			$this->maniaControl->getCommandManager()->registerCommandListener('rank', $this->ranking, 'command_showRank', false, 'Shows your current ServerRank.');
			$this->maniaControl->getCommandManager()->registerCommandListener('nextrank', $this->ranking, 'command_nextRank', false, 'Shows the person in front of you in the ServerRanking.');
			$this->maniaControl->getCommandManager()->registerCommandListener(array('topranks', 'top100'), $this, 'command_topRanks', false, 'Shows an overview of the best-ranked 100 players.');

			$this->maniaControl->getCommandManager()->registerCommandListener('summary', $this, 'command_summary', false, 'Shows a summary of your current records.');
		}

		if (!$this->records) {
			$this->maniaControl->getChat()->sendError('[ExtendedServerRankingPlugin] You need to activate ' . self::DEFAULT_LOCAL_RECORDS_PLUGIN);
		} else {
		}

		$this->maniaControl->getManialinkManager()->registerManialinkPageAnswerListener(self::ACTION_OPEN_RANKINGS, $this, 'command_topRanks');
		$itemQuad = new Quad_UIConstruction_Buttons();
		$itemQuad->setSubStyle($itemQuad::SUBSTYLE_Stats);
		$itemQuad->setAction(self::ACTION_OPEN_RANKINGS);
		$this->maniaControl->getActionsMenu()->addMenuItem($itemQuad, true, 11, 'Open Rankings');

	}


	/**
	 * @see \ManiaControl\Plugins\Plugin::unload()
	 */
	public function unload() {
	}

	/**
	 * Handles /topranks|top100 command
	 *
	 * @param array  $chatCallback
	 * @param Player $player
	 */
	public function command_topRanks(array $chatCallback, Player $player) {
		$this->showTopRanksList($player);
	}

	/**
	 * Provide a ManiaLink window with the top ranks to the player
	 *
	 * @param Player $player
	 */
	private function showTopRanksList(Player $player) {
		if (!$this->ranking || !$this->records) {
			return;
		}

		$requiredRecords = $this->maniaControl->getSettingManager()->getSettingValue($this, ServerRankingPlugin::SETTING_MIN_REQUIRED_RECORDS);
		$maxRecords = $this->maniaControl->getSettingManager()->getSettingValue($this, ServerRankingPlugin::SETTING_MAX_STORED_RECORDS);

		$mysqli = $this->maniaControl->getDatabase()->getMysqli();
		
		$players = array();
		
		$maps = $this->maniaControl->getMapManager()->getMaps();
		foreach ($maps as $map) {
			$allRecords = $this->records->getLocalRecords($map, $maxRecords);
			$index = 1;
			foreach ($allRecords as $record) {
				if (isset($players[$record->playerIndex])) {
					$players[$record->playerIndex][0]++;
					if ($index == 1) { 
						$players[$record->playerIndex][1]++;
					}
					if ($index < $players[$record->playerIndex][2]) {
						$players[$record->playerIndex][2] = $index;
					}
				} else {
					$topone = 0;
					if ($index == 1) {
						$topone = 1;
					}

					$players[$record->playerIndex] = array(1, $topone, $index);
				}
				$index++;
			}
		}

		$queryRank  = "SELECT * FROM `" . ServerRankingPlugin::TABLE_RANK . "`
					ORDER BY `Rank` ASC LIMIT 0, 100;";

		$resultRank = $mysqli->query($queryRank);
		if ($mysqli->error) {
			trigger_error($mysqli->error);
			return;
		}

		$width  = $this->maniaControl->getManialinkManager()->getStyleManager()->getListWidgetsWidth();
		$height = $this->maniaControl->getManialinkManager()->getStyleManager()->getListWidgetsHeight();

		// create manialink
		$maniaLink = new ManiaLink(ManialinkManager::MAIN_MLID);
		$script    = $maniaLink->getScript();
		$paging    = new Paging();
		$script->addFeature($paging);

		// Main frame
		$frame = $this->maniaControl->getManialinkManager()->getStyleManager()->getDefaultListFrame($script, $paging);
		$maniaLink->addChild($frame);

		// Start offsets
		$posX = -$width / 2;
		$posY = $height / 2;

		//Predefine description Label
		$descriptionLabel = $this->maniaControl->getManialinkManager()->getStyleManager()->getDefaultDescriptionLabel();
		$frame->addChild($descriptionLabel);

		// Headline
		$headFrame = new Frame();
		$frame->addChild($headFrame);
		$headFrame->setY($posY - 5);
		$array = array('$oRank' => $posX + 5, '$oNickname' => $posX + 18, '$oAverage' => $posX + 70, '$oRecords' => $posX + 90, '$oTop 1s' => $posX + 105, '$oBest' => $posX + 120);
		$this->maniaControl->getManialinkManager()->labelLine($headFrame, $array);

		$index = 1;
		$posY -= 10;
		$pageFrame = null;

		while ($rankedPlayer = $resultRank->fetch_object()) {
			if ($index % 15 === 1) {
				$pageFrame = new Frame();
				$frame->addChild($pageFrame);
				$posY = $height / 2 - 10;
				$paging->addPageControl($pageFrame);
			}

			$playerFrame = new Frame();
			$pageFrame->addChild($playerFrame);
			$playerFrame->setY($posY);

			if ($rankedPlayer->PlayerIndex == $player->index) {
				$currentQuad = new Quad_Icons64x64_1();
				$playerFrame->addChild($currentQuad);
				$currentQuad->setX($posX + 3.5);
				$currentQuad->setSize(4, 4);
				$currentQuad->setSubStyle($currentQuad::SUBSTYLE_ArrowBlue);
			}

/*			if ($index % 2 !== 0) {
				$lineQuad = new Quad_BgsPlayerCard();
				$playerFrame->addChild($lineQuad);
				$lineQuad->setSize($width, 4);
				$lineQuad->setSubStyle($lineQuad::SUBSTYLE_BgPlayerCardBig);
				$lineQuad->setZ(0.001);
			}*/

			$playerObject = $this->maniaControl->getPlayerManager()->getPlayerByIndex($rankedPlayer->PlayerIndex);

			$labelLine = new LabelLine($playerFrame);
			$labelLine->addLabelEntryText($rankedPlayer->Rank, $posX + 5, 10);
			$labelLine->addLabelEntryText($playerObject->nickname, $posX + 18, 50);
			$labelLine->addLabelEntryText((string) round($rankedPlayer->Avg, 2), $posX + 70, 15);
			if (isset($players[$rankedPlayer->PlayerIndex])) {
				$labelLine->addLabelEntryText($players[$rankedPlayer->PlayerIndex][0], $posX + 90, 10);
				$labelLine->addLabelEntryText($players[$rankedPlayer->PlayerIndex][1], $posX + 105, 10);
				$labelLine->addLabelEntryText($players[$rankedPlayer->PlayerIndex][2], $posX + 120, 10);
			}
			$labelLine->render();

			$posY -= 4;
			$index++;
		}

		// Render and display xml
		$this->maniaControl->getManialinkManager()->displayWidget($maniaLink, $player, 'TopRanks');
		$resultRank->free_result();
	}

	/**
	 * Handles /summary command
	 *
	 * @param array  $chatCallback
	 * @param Player $player
	 */
	public function command_summary(array $chatCallback, Player $player) {
		$this->showSummary($player);
	}
	
	/**
	 * Provide a ManiaLink window with the top ranks to the player
	 *
	 * @param Player $player
	 */
	private function showSummary(Player $player) {
		if (!$this->records) {
			return;
		}

		$maxRecords = $this->maniaControl->getSettingManager()->getSettingValue($this, ServerRankingPlugin::SETTING_MAX_STORED_RECORDS);
		
		$maps = $this->maniaControl->getMapManager()->getMaps();
		$summary = array();
		foreach ($maps as $map) {
			$allRecords = $this->records->getLocalRecords($map, $maxRecords);
			$index = 1;
			foreach ($allRecords as $record) {
				if ($record->playerIndex == $player->index) {
					if (isset($summary[$index])) {
						$summary[$index]++;
					} else {
						$summary[$index] = 1;
					}
					break;
				}
				$index++;
			}
		}
		
		ksort($summary, SORT_NUMERIC);
		$sum = 0;
		$best = array();
		foreach ($summary as $rank => $number) {
			$sum += $number;
		}
		
		$message = $player->nickname . ' $z$0f3has $<$fff' . $sum . '$> Local records, the best 3 being: ';
		$index = 1;
		foreach ($summary as $rank => $number) {
			if ($index >= 3) {
				break;
			}
			$message = $message . ' $<$fff' . $number . '$> rec#$<$ff3' . $rank . '$>';
			if ($index < 3) {
				$message = $message . ', ';
			}
			$index++;
		}
		$this->maniaControl->getChat()->sendChat($message, $player);
	}
	
}

